"""
Misc ASCII art
"""

WARNING = r"""
               _mBma
              sQf "QL
             jW(   -$g.
            jW'     -$m,
          .y@'  _aa.  4m,
         .mD`  ]QQWQ.  4Q,
        _mP`   ]QQQQ    ?Q/
       _QF     )WQQ@     ?Qc
      <QF       QQQF      )Qa
     jW(        QQQf       "QL
    jW'         ]H8'        -Q6.
  .y@'          _as.         -$m.
 .m@`          ]QQWQ.         -4m,
_mP`           -?$8!            4Q,
mE                               $m
?$gyygggggggggwywgyygggggggygggggD(
"""
